@@ -52,7 +52,7 @@ static xchat_hook	*pm_dialog;
 static xchat_hook	*ch_nick;
 static xchat_hook	*tab_focus;
 static GtkWindow	*win;
-static gchar 		*focused;
+static gchar		*focused;
 static gboolean		run = FALSE;
 static void hide_indicator (const gchar *channel, gpointer indicator, gpointer data);
 static void remove_indicator (const gchar *channel, gpointer indicator);
@@ -79,8 +79,8 @@ indicator_display (gpointer indicator, guint timestamp, const gchar *channel)
 			/*
 			g_debug ("xchat-indicator: Showing window");
 			*/
-			xchat_command (ph, "GUI SHOW"); 
-			xchat_command (ph, "GUI FOCUS"); 
+			xchat_command (ph, "GUI SHOW");
+			xchat_command (ph, "GUI FOCUS");
 			gtk_widget_show (GTK_WIDGET (win));
 			really_activate_window (GTK_WINDOW (win));
 			/* gtk_window_present_with_time(GTK_WINDOW (win), timestamp); */
@@ -102,7 +102,7 @@ server_display (IndicateServer * indicate_server, guint timestamp, gpointer data
 {
 	GtkWindow *win = (GtkWindow *)xchat_get_info (ph, "win_ptr");
         really_activate_window (GTK_WINDOW (win));
-	xchat_command (ph, "GUI SHOW"); 
+	xchat_command (ph, "GUI SHOW");
 	/* gtk_window_present_with_time(GTK_WINDOW (win), timestamp); */
 	/*
 	g_debug ("xchat-indicator: Showing the window");
@@ -110,7 +110,7 @@ server_display (IndicateServer * indicate_server, guint timestamp, gpointer data
         /* XXX: Should clear all indicators */
 }
 
-static void 
+static void
 hide_indicator (const gchar *channel, gpointer indicator, gpointer data)
 {
 	/*
@@ -122,7 +122,11 @@ hide_indicator (const gchar *channel, gpointer indicator, gpointer data)
 static void
 remove_indicator (const gchar *channel, gpointer indicator)
 {
-	if (indicator == g_hash_table_lookup (indicators, channel))
+	gpointer orig_channel, orig_indicator;
+
+	if (g_hash_table_lookup_extended (indicators, channel,
+				&orig_channel, &orig_indicator)
+			&& orig_indicator == indicator)
 	{
 		/*
 		g_debug ("xchat-indicator: removing indicator for channel %s", channel);
@@ -130,6 +134,7 @@ remove_indicator (const gchar *channel, gpointer indicator)
 		indicate_indicator_hide (INDICATE_INDICATOR (indicator));
 		g_hash_table_remove (indicators, channel);
 		g_object_unref (indicator);
+		g_free (orig_channel);
 	}
 #ifdef HAVE_UNITY
         if (launcher == NULL)
@@ -179,8 +184,8 @@ add_indicator (const gchar *channel)
 {
 	IndicateIndicator *indicator = NULL;
 	GTimeVal time;
-	gpointer chan = channel;
-	
+	gpointer chan;
+
 	indicator = g_hash_table_lookup (indicators, channel);
 	if (indicator != NULL) {
 		update_indicator (indicator);
@@ -192,7 +197,7 @@ add_indicator (const gchar *channel)
 		GtkWindow *win = (GtkWindow *)xchat_get_info (ph, "win_ptr");
 		if (GTK_IS_WINDOW (win))
 		{
-        		g_signal_connect(G_OBJECT(win), "notify::has-toplevel-focus", G_CALLBACK (focus_win_cb), NULL);
+			g_signal_connect(G_OBJECT(win), "notify::has-toplevel-focus", G_CALLBACK (focus_win_cb), NULL);
 		run = TRUE;
 		}
 	}
@@ -205,6 +210,8 @@ add_indicator (const gchar *channel)
 	indicate_indicator_set_property (INDICATE_INDICATOR (indicator),
 		"draw-attention", "true");
 
+	chan = g_strdup (channel);
+
 	g_signal_connect (G_OBJECT (indicator),
 		INDICATE_INDICATOR_SIGNAL_DISPLAY,
 		G_CALLBACK (indicator_display), chan);
@@ -234,25 +241,32 @@ add_indicator (const gchar *channel)
 #endif
 }
 
-/*
 static int
 nick_change_cb (char *word[], void *data)
 {
-        gpointer indicator;
+	gpointer indicator, key;
+	gboolean indicator_present;
         xchat_context *ctx;
-        ctx = xchat_find_context (ph, NULL, word[2]);
-        xchat_set_context (ph, ctx);
-        const gchar *channel = xchat_get_info (ph, "channel");
+	char *old_nick, *new_nick;
 
-        g_debug ("xchat-indicator: Checking for indicators for %s", channel);
-        indicator = g_hash_table_lookup (indicators, channel);
-        if (indicator != NULL) {
-                g_debug ("xchat-indicator: Found indicator for %s, changing it to %s", word[1], word[2]);
-                indicate_indicator_set_property (INDICATE_INDICATOR (indicator),
-                        "sender", channel);
-        }
+	old_nick = word[1];
+	new_nick = word[2];
+
+	indicator_present = g_hash_table_lookup_extended (indicators,
+			old_nick, &key, &indicator);
+
+	if (!indicator_present)
+		return XCHAT_EAT_NONE;
+
+	g_hash_table_remove (indicators, old_nick);
+	g_free (key);
+	key = g_strdup (new_nick);
+	g_hash_table_insert (indicators, key, indicator);
+	indicate_indicator_set_property (INDICATE_INDICATOR (indicator),
+		"sender", new_nick);
+
+	return XCHAT_EAT_NONE;
 }
-*/
 
 static int
 indicate_msg_cb (char *word[], gpointer data)
@@ -399,10 +413,8 @@ xchat_plugin_init (xchat_plugin *plugin_handle, char **plugin_name, char **plugi
 		XCHAT_PRI_NORM, indicate_msg_cb, NULL);
 	tab_focus = xchat_hook_print (ph, "Focus Tab",
 		XCHAT_PRI_NORM, focus_tab_cb, NULL);
-	/*
         ch_nick = xchat_hook_print (ph, "Change Nick",
                 XCHAT_PRI_NORM, nick_change_cb, NULL);
-	*/
 
 	xchat_print (ph,  (g_strjoin (" ", MESSAGING_INDICATOR_PLUGIN_NAME, MESSAGING_INDICATOR_PLUGIN_VERSION, _("plugin loaded."), NULL)));
 
@@ -429,6 +441,7 @@ xchat_plugin_deinit (void)
 		g_debug ("xchat-indicator: removing indicator for channel %p", channel);
 		*/
 		indicate_indicator_hide (INDICATE_INDICATOR (indicator));
+		g_free (channel);
 	}
 
 
